@@ -18,7 +18,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
 #include "PYPBopomofoEditor.h"
+#include <assert.h>
 #include "PYConfig.h"
 #include "PYLibPinyin.h"
 #include "PYPinyinProperties.h"
@@ -40,29 +42,29 @@ const static gchar * bopomofo_select_keys[] = {
     "qweasdzxcr"
 };
 
-LibPinyinBopomofoEditor::LibPinyinBopomofoEditor
+BopomofoEditor::BopomofoEditor
 (PinyinProperties & props, Config & config)
-    : LibPinyinPhoneticEditor (props, config),
+    : PhoneticEditor (props, config),
       m_select_mode (FALSE)
 {
     m_instance = LibPinyinBackEnd::instance ().allocChewingInstance ();
 }
 
-LibPinyinBopomofoEditor::~LibPinyinBopomofoEditor (void)
+BopomofoEditor::~BopomofoEditor (void)
 {
     LibPinyinBackEnd::instance ().freeChewingInstance (m_instance);
     m_instance = NULL;
 }
 
 void
-LibPinyinBopomofoEditor::reset (void)
+BopomofoEditor::reset (void)
 {
     m_select_mode = FALSE;
-    LibPinyinPhoneticEditor::reset ();
+    PhoneticEditor::reset ();
 }
 
 gboolean
-LibPinyinBopomofoEditor::insert (gint ch)
+BopomofoEditor::insert (gint ch)
 {
     /* is full */
     if (G_UNLIKELY (m_text.length () >= MAX_PINYIN_LEN))
@@ -77,7 +79,7 @@ LibPinyinBopomofoEditor::insert (gint ch)
 
 
 gboolean
-LibPinyinBopomofoEditor::processGuideKey (guint keyval, guint keycode,
+BopomofoEditor::processGuideKey (guint keyval, guint keycode,
                                           guint modifiers)
 {
     if (!m_config.guideKey ())
@@ -99,7 +101,7 @@ LibPinyinBopomofoEditor::processGuideKey (guint keyval, guint keycode,
 }
 
 gboolean
-LibPinyinBopomofoEditor::processAuxiliarySelectKey
+BopomofoEditor::processAuxiliarySelectKey
 (guint keyval, guint keycode, guint modifiers)
 {
     if (G_UNLIKELY (cmshm_filter (modifiers) != 0))
@@ -135,7 +137,7 @@ LibPinyinBopomofoEditor::processAuxiliarySelectKey
 }
 
 gboolean
-LibPinyinBopomofoEditor::processSelectKey (guint keyval, guint keycode,
+BopomofoEditor::processSelectKey (guint keyval, guint keycode,
                                            guint modifiers)
 {
     if (G_UNLIKELY (!m_text))
@@ -164,14 +166,16 @@ LibPinyinBopomofoEditor::processSelectKey (guint keyval, guint keycode,
 }
 
 gboolean
-LibPinyinBopomofoEditor::processBopomofo (guint keyval, guint keycode,
+BopomofoEditor::processBopomofo (guint keyval, guint keycode,
                                           guint modifiers)
 {
     if (G_UNLIKELY (cmshm_filter (modifiers) != 0))
         return m_text ? TRUE : FALSE;
 
-    if (!(pinyin_in_chewing_keyboard (m_instance, keyval, NULL)))
+    gchar ** symbols = NULL;
+    if (!pinyin_in_chewing_keyboard (m_instance, keyval, &symbols))
         return FALSE;
+    g_strfreev (symbols);
 
     if (keyval == IBUS_space)
         return FALSE;
@@ -182,7 +186,7 @@ LibPinyinBopomofoEditor::processBopomofo (guint keyval, guint keycode,
 }
 
 gboolean
-LibPinyinBopomofoEditor::processKeyEvent (guint keyval, guint keycode,
+BopomofoEditor::processKeyEvent (guint keyval, guint keycode,
                                           guint modifiers)
 {
     modifiers &= (IBUS_SHIFT_MASK |
@@ -214,7 +218,7 @@ LibPinyinBopomofoEditor::processKeyEvent (guint keyval, guint keycode,
     case IBUS_Page_Down: case IBUS_KP_Page_Down:
     case IBUS_Tab:
         m_select_mode = TRUE;
-        return LibPinyinPhoneticEditor::processFunctionKey
+        return PhoneticEditor::processFunctionKey
             (keyval, keycode, modifiers);
 
     case IBUS_BackSpace:
@@ -224,18 +228,18 @@ LibPinyinBopomofoEditor::processKeyEvent (guint keyval, guint keycode,
     case IBUS_Home:      case IBUS_KP_Home:
     case IBUS_End:       case IBUS_KP_End:
         m_select_mode = FALSE;
-        return LibPinyinPhoneticEditor::processFunctionKey
+        return PhoneticEditor::processFunctionKey
             (keyval, keycode, modifiers);
 
     default:
-        return LibPinyinPhoneticEditor::processFunctionKey
+        return PhoneticEditor::processFunctionKey
             (keyval, keycode, modifiers);
     }
     return FALSE;
 }
 
 void
-LibPinyinBopomofoEditor::updatePinyin (void)
+BopomofoEditor::updatePinyin (void)
 {
     if (G_UNLIKELY (m_text.empty ())) {
         m_pinyin_len = 0;
@@ -251,7 +255,7 @@ LibPinyinBopomofoEditor::updatePinyin (void)
 }
 
 void
-LibPinyinBopomofoEditor::commit ()
+BopomofoEditor::commit ()
 {
     if (G_UNLIKELY (m_text.empty ()))
         return;
@@ -273,9 +277,11 @@ LibPinyinBopomofoEditor::commit ()
     /* text after pinyin */
     const gchar *p = m_text.c_str() + m_pinyin_len;
     while (*p != '\0') {
-        const char * symbol = NULL;
-        if (pinyin_in_chewing_keyboard(m_instance, *p, &symbol)) {
-            m_buffer << symbol;
+        gchar ** symbols = NULL;
+        if (pinyin_in_chewing_keyboard (m_instance, *p, &symbols)) {
+            assert (1 == g_strv_length (symbols));
+            m_buffer << symbols[0];
+            g_strfreev (symbols);
         } else {
             if (G_UNLIKELY (m_props.modeFull ())) {
                 m_buffer.appendUnichar (HalfFullConverter::toFull (*p));
@@ -287,13 +293,15 @@ LibPinyinBopomofoEditor::commit ()
     }
 
     pinyin_train(m_instance);
+    if (m_config.rememberEveryInput ())
+        LibPinyinBackEnd::instance ().rememberUserInput (m_instance);
     LibPinyinBackEnd::instance ().modified();
-    LibPinyinPhoneticEditor::commit ((const gchar *)m_buffer);
+    PhoneticEditor::commit ((const gchar *)m_buffer);
     reset();
 }
 
 void
-LibPinyinBopomofoEditor::updatePreeditText ()
+BopomofoEditor::updatePreeditText ()
 {
     /* preedit text = guessed sentence + un-parsed pinyin text */
     if (G_UNLIKELY (m_text.empty ())) {
@@ -327,7 +335,7 @@ LibPinyinBopomofoEditor::updatePreeditText ()
 }
 
 void
-LibPinyinBopomofoEditor::updateAuxiliaryText (void)
+BopomofoEditor::updateAuxiliaryText (void)
 {
     if (G_UNLIKELY (m_text.empty ())) {
         hideAuxiliaryText ();
@@ -351,7 +359,7 @@ LibPinyinBopomofoEditor::updateAuxiliaryText (void)
 
         gchar * str = NULL;
         if (G_UNLIKELY (cursor == m_cursor)) { /* at word boundary. */
-            pinyin_get_chewing_string (m_instance, key, &str);
+            pinyin_get_zhuyin_string (m_instance, key, &str);
             m_buffer << '|' << str;
             g_free (str);
         } else if (G_LIKELY (cursor < m_cursor &&
@@ -366,22 +374,28 @@ LibPinyinBopomofoEditor::updateAuxiliaryText (void)
             String before = raw.substr (0, offset);
             String after = raw.substr (offset);
             String::const_iterator iter;
-            const char * symbol = NULL;
-            for ( iter = before.begin (); iter != before.end (); ++iter) {
-                if ( pinyin_in_chewing_keyboard(m_instance, *iter, &symbol))
-                    m_buffer << symbol;
-                else
+            gchar ** symbols = NULL;
+            for (iter = before.begin (); iter != before.end (); ++iter) {
+                if (pinyin_in_chewing_keyboard(m_instance, *iter, &symbols)) {
+                    assert (1 == g_strv_length (symbols));
+                    m_buffer << symbols[0];
+                    g_strfreev (symbols);
+                } else {
                     m_buffer << *iter;
+                }
             }
             m_buffer << '|';
-            for ( iter = after.begin (); iter != after.end (); ++iter) {
-                if ( pinyin_in_chewing_keyboard (m_instance, *iter, &symbol))
-                    m_buffer << symbol;
-                else
+            for (iter = after.begin (); iter != after.end (); ++iter) {
+                if (pinyin_in_chewing_keyboard (m_instance, *iter, &symbols)) {
+                    assert (1 == g_strv_length (symbols));
+                    m_buffer << symbols[0];
+                    g_strfreev (symbols);
+                } else {
                     m_buffer << *iter;
+                }
             }
         } else { /* other words */
-            pinyin_get_chewing_string (m_instance, key, &str);
+            pinyin_get_zhuyin_string (m_instance, key, &str);
             m_buffer << ' ' << str;
             g_free (str);
         }
