@@ -28,18 +28,17 @@
 
 namespace PY {
 
-class LibPinyinBopomofoEngine : public Engine {
+class BopomofoEngine : public Engine {
 public:
-    LibPinyinBopomofoEngine (IBusEngine *engine);
-    ~LibPinyinBopomofoEngine (void);
+    BopomofoEngine (IBusEngine *engine);
+    ~BopomofoEngine (void);
 
     // virtual functions
+    gboolean processAccelKeyEvent (guint keyval, guint keycode,
+                                   guint modifiers);
     gboolean processKeyEvent (guint keyval, guint keycode, guint modifiers);
     void focusIn (void);
     void focusOut (void);
-#if IBUS_CHECK_VERSION (1, 5, 4)
-    void setContentType (guint purpose, guint hints);
-#endif
     void reset (void);
     void enable (void);
     void disable (void);
@@ -63,10 +62,6 @@ private:
 private:
     PinyinProperties m_props;
 
-#if IBUS_CHECK_VERSION (1, 5, 4)
-    IBusInputPurpose m_input_purpose;
-#endif
-
     guint m_prev_pressed_key;
 
     enum {
